package org.briarproject.onionwrapper;

import org.briarproject.nullsafety.NotNullByDefault;

import java.util.List;

import javax.annotation.concurrent.ThreadSafe;

@ThreadSafe
@NotNullByDefault
public interface CircumventionProvider {

	enum BridgeType {

		DEFAULT_OBFS4("d"),
		NON_DEFAULT_OBFS4("n"),
		VANILLA("v"),
		MEEK("m"),
		SNOWFLAKE("s");

		final String letter;

		BridgeType(String letter) {
			this.letter = letter;
		}
	}

	/**
	 * Countries where default obfs4 bridges should be used.
	 */
	String[] COUNTRIES_DEFAULT_BRIDGES = {"BY"};

	/**
	 * Countries where non-default obfs4 and vanilla bridges should be used.
	 */
	String[] COUNTRIES_NON_DEFAULT_BRIDGES = {"BY", "CN", "EG", "HK", "IR", "RU"};

	/**
	 * Countries where meek bridges should be used.
	 */
	String[] COUNTRIES_MEEK_BRIDGES = {"TM"};

	/**
	 * Countries where snowflake bridges should be used.
	 */
	String[] COUNTRIES_SNOWFLAKE_BRIDGES = {"BY", "CN", "EG", "HK", "IR", "RU", "TM"};

	/**
	 * Returns the types of bridge connection that are suitable for the given country, or
	 * {@link BridgeType#DEFAULT_OBFS4} and {@link BridgeType#VANILLA} if we don't have any
	 * specific recommendations for the given country.
	 */
	List<BridgeType> getSuitableBridgeTypes(String countryCode);

	/**
	 * Returns bridges of the given type that are usable in the given country.
	 *
	 * @param letsEncrypt Specifies whether the client is able to verify Let's Encrypt TLS
	 * 		certificates signed with the IdentTrust DST Root X3 certificate. Versions of Android
	 * 		older than 7.1 consider the certificate to have expired at the end of September 2021.
	 * 		This parameter is currently ignored, as no domain-fronted bridges use Let's Encrypt
	 * 		certificates.
	 */
	List<String> getBridges(BridgeType type, String countryCode, boolean letsEncrypt);

	/**
	 * Returns bridges of the given type that are usable in the given country.
	 */
	List<String> getBridges(BridgeType type, String countryCode);
}
